#ifndef Lib_BigVector_h
#define Lib_BigVector_h 

// Compared to std::vector, BigVector gains 
// at fill time, but wastes time when clearing (then deleting).

#include <vector>

namespace Lib {

template <class T> class BigVector {
public:
  BigVector(unsigned int aChunck = 128)
  :fChunck(aChunck),fAllocated(0),fSize(0){
    if(fChunck<=0) fChunck = 128;
  }
  BigVector(const BigVector<T>& aFrom)
  :fChunck(aFrom.fChunck),fAllocated(0),fSize(0) {
    unsigned int fromSize = aFrom.fSize;
    for(unsigned int findex=0;findex<fromSize;findex++) {  
      unsigned int ichunck = findex / aFrom.fChunck;
      unsigned int ipos = findex % aFrom.fChunck;
      push_back((*(aFrom.fData[ichunck]))[ipos]);
    }
  }
  virtual ~BigVector() { clear();}
  BigVector<T>& operator=(const BigVector<T>& aFrom) {
    clear();
    fChunck = aFrom.fChunck;
    unsigned int fromSize = aFrom.fSize;
    for(unsigned int findex=0;findex<fromSize;findex++) {  
      unsigned int ichunck = findex / aFrom.fChunck;
      unsigned int ipos = findex % aFrom.fChunck;
      push_back((*(aFrom.fData[ichunck]))[ipos]);
    }
    return *this;
  }
public:
  unsigned int size() const { return fSize;}
  const T& operator[](unsigned int aIndex) const {
    unsigned int ichunck = aIndex / fChunck;
    unsigned int ipos = aIndex % fChunck;
    return (*(fData[ichunck]))[ipos];
  }
  T& operator[](unsigned int aIndex) {
    unsigned int ichunck = aIndex / fChunck;
    unsigned int ipos = aIndex % fChunck;
    return (*(fData[ichunck]))[ipos];
  }
  void push_back(const T& aValue) {
    if( fSize >= fAllocated) {
      fData.push_back(new std::vector<T>(fChunck));
      fAllocated += fChunck;
    }
    unsigned int ichunck = fSize / fChunck;
    unsigned int ipos = fSize % fChunck;
    (*(fData[ichunck]))[ipos] = aValue;
    fSize++;
  }
  void clear() {
    unsigned int number = fData.size();
    for(unsigned int index=0;index<number;index++) delete fData[index];
    fData.clear();
    fAllocated = 0;
    fSize = 0;    
  }
  bool value(unsigned int aIndex,T& aValue) const {
    if(aIndex>=fSize) {
      //FIXME aValue = fDefault;
      return false;
    } else {
      unsigned int ichunck = aIndex / fChunck;
      unsigned int ipos = aIndex % fChunck;
      aValue = (*(fData[ichunck]))[ipos];
      return true;
    }
  }
  void valueNoCheck(unsigned int aIndex,T& aValue) const {
    unsigned int ichunck = aIndex / fChunck;
    unsigned int ipos = aIndex % fChunck;
    aValue = (*(fData[ichunck]))[ipos];
  }
private:
  unsigned int fChunck;
  std::vector< std::vector<T>* > fData;
  unsigned int fAllocated;
  unsigned int fSize;
};

}

#endif
