#ifndef Slash_Tools_UI_h
#define Slash_Tools_UI_h

#include <Slash/Tools/Managers.h>
#include <Slash/UI/IUI.h>

namespace Slash {

inline std::string s_default() {return "default";}
inline std::string s_current() {return "@current@";}

inline Slash::UI::IUI* find_UI(Slash::Core::ISession& aSession,const std::string& aName = s_default(),bool aWarn = false) {
  Slash::UI::IUI_Manager* mgr = Slash::uiManager(aSession);
  if(!mgr) {
    if(aWarn) {
      aSession.cout() << "Slash::find_UI :"
                      << " UI_Manager not found." 
                      << std::endl;
    }
    return 0;
  }
  return mgr->find(aName);
}

}

#include "smanip.h"

namespace Slash {

inline Slash::UI::IWidget* find_widget(Slash::UI::IUI& aUI,const std::string& aString) {
  Slash::UI::IWidget* widget = 0;
  if(aString==s_current()) {
    widget = aUI.currentWidget();
    if(!widget) {
      std::ostream& out = aUI.session().cout();
      out << "Slash::find_widget : "
          << " no current widget."
          << std::endl;
      return 0;
    }
  } else {
    widget = aUI.findWidget(aString);
    if(!widget) {
      std::ostream& out = aUI.session().cout();
      out << "Slash::find_widget : "
          << sout(aString) << " not found."
          << std::endl;
      return 0;
    }
  }
  return widget;
}

template <class T>
inline bool set_parameter(Slash::UI::IUI& aUI,const std::string& aWhat,T aValue){return aUI.setParameter(aWhat,to<T>(aValue));}

inline bool set_parameter(Slash::UI::IUI& aUI,const std::string& aWhat,bool aValue){return aUI.setParameter(aWhat,std::string(aValue?"true":"false"));}

template <class T>
inline bool parameter(Slash::UI::IUI& aUI,const std::string& aWhat,T& aValue){
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) {
    std::ostream& out = aUI.session().cout();
    out << "Paramater " << Slash::sout(aWhat) << " not found."
        << std::endl;
    aValue = 0;
    return false;
  }
  if(!Slash::to<T>(s,aValue)) {
    std::ostream& out = aUI.session().cout();
    out << "Passed value " << Slash::sout(s)
        << " for parameter " << Slash::sout(aWhat) 
        << " is of bad type."
        << std::endl;
    return false;
  }
  return true;
}

inline bool parameter(Slash::UI::IUI& aUI,const std::string& aWhat,std::string& aValue){
  if(!aUI.parameterValue(aWhat,aValue)) {
    std::ostream& out = aUI.session().cout();
    out << "Paramater " << Slash::sout(aWhat) << " not found."
        << std::endl;
    aValue = "";
    return false;
  }
  return true;
}

inline bool parameter(Slash::UI::IUI& aUI,const std::string& aWhat,bool& aValue){
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) {
    std::ostream& out = aUI.session().cout();
    out << "Paramater " << Slash::sout(aWhat) << " not found."
        << std::endl;
    aValue = false;
    return false;
  }
  if(!Slash::to(s,aValue)) {
    std::ostream& out = aUI.session().cout();
    out << "Passed value " << Slash::sout(s)
        << " for parameter " << Slash::sout(aWhat) 
        << " not a boolean."
        << std::endl;
    return false;
  }
  return true;
}


}

//////////////////////////////////////////////////////////////////////////////
/// ISoViewer ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>

#include "cast.h"

namespace Slash {

inline Slash::UI::ISoViewer* find_SoViewer(Slash::UI::IUI& aUI,const std::string& aString) {
  Slash::UI::IWidget* widget = find_widget(aUI,aString);
  if(!widget) return 0;
  Slash::UI::ISoViewer* soViewer = 
    cast<Slash::UI::ISoViewer,Slash::UI::IWidget>
      (*widget,"Slash::UI::ISoViewer");
  if(!soViewer) {
    std::ostream& out = aUI.session().cout();
    out << "Slash::find_SoViewer : "
        << sout(aString) << " not a Slash::UI::ISoViewer."
        << std::endl;
    return 0;
  }
  return soViewer;
}

inline Slash::UI::ISoViewer* cast_SoViewer(Slash::UI::IWidget& a_widget){
  return cast<Slash::UI::ISoViewer,Slash::UI::IWidget>
      (a_widget,"Slash::UI::ISoViewer");
}


}

#endif
